export class SHA256 {
  /** @param {string} message */
  static async fromString(message) {
    const encoder = new TextEncoder();
    const data = encoder.encode(message);
    return await crypto.subtle.digest("SHA-256", data);
  }

  /** @param {ArrayBuffer} buffer */
  static async fromBuffer(buffer) {
    return await crypto.subtle.digest("SHA-256", buffer);
  }

  /** @param {ArrayBuffer} buffer */
  static toHex(buffer) {
    const hashArray = Array.from(new Uint8Array(buffer));
    return hashArray.map((b) => b.toString(16).padStart(2, "0")).join("");
  }

  /** @param {number} code */
  static identify(code) {
    // ES256
    if (code === -7) {
      return {
        hash: { name: "SHA-256" },
        name: "ECDSA",
        namedCurve: "P-256",
      };
    }

    // RS256
    if (code === -257) {
      return {
        hash: { name: "SHA-256" },
        name: "RSASSA-PKCS1-v1_5",
      };
    }

    throw Error("Algorithm code must be one of: -7, -257");
  }

  /**
   * Convert DER signature to raw for webcrypto
   * @param {ArrayBuffer} signature
   */
  static der2raw(signature) {
    const array = new Uint8Array(signature);
    const rStart = array[4] === 0 ? 5 : 4;
    const rEnd = rStart + 32;
    const sStart = array[rEnd + 2] === 0 ? rEnd + 3 : rEnd + 2;

    return new Uint8Array([
      ...array.slice(rStart, rEnd),
      ...array.slice(sStart),
    ]);
  }
}
