/**
 * AuthenticatorData structure: https://www.w3.org/TR/webauthn-2/#sctn-authenticator-data
 * @param {ArrayBuffer} value - binary authenticator data
 * @returns {import('../types').AuthenticatorData}
 */
export function parseAuthenticatorData(value) {
  if (value.byteLength < 37) {
    throw Error("Authenticator Data must be a byte array of 37 bytes");
  }

  const rpId = value.slice(0, 32);

  const flagsInt = new DataView(value.slice(32, 33)).getUint8(0);

  /** @type {import('../types').AuthenticatorDataFlags} */
  const flags = {
    up: !!(flagsInt & 1),
    uv: !!(flagsInt & 4),
    at: !!(flagsInt & 64),
    ed: !!(flagsInt & 128),
  };

  const counter = new DataView(value.slice(33, 37)).getUint32(0, false);

  return {
    rpId,
    flags,
    counter,
  };
}
