export interface ClientDataJSON {
  type: "webauthn.create" | "webauthn.get";
  challenge: string;
  origin: string;
}

export interface AuthenticatorDataFlags {
  up: boolean; // user presence
  uv: boolean; // user verified
  at: boolean; // attested credential data present
  ed: boolean; // extension data present
}

export interface AuthenticatorData {
  rpId: ArrayBuffer;
  flags: AuthenticatorDataFlags;
  counter: number;
}

export interface Credential {
  id: string;
  publicKey: string;
  algorithm: number;
  counter: number;
}

export namespace Registration {
  export interface Settings {
    rpId: string;
    rpName: string;
    userId: string;
    userName: string;
    excludeCredentialIds?: string[];
  }

  export interface PublicKeyOptionsJSON {
    challenge: string;
    rp: PublicKeyCredentialRpEntity;
    user: { id: string; name: string; displayName: string };
    pubKeyCredParams: PublicKeyCredentialParameters[];
    authenticatorSelection: AuthenticatorSelectionCriteria;
    timeout: number;
    attestation: AttestationConveyancePreference;
    excludeCredentials?: { type: "public-key"; id: string }[];
  }

  export interface AttestationJSON {
    credentialId: string;
    credentialType: string;
    clientDataJSON: string;
    publicKey: string;
    algorithm: number;
    authenticatorData: string;
  }

  export interface VerificationOptions {
    expectedChallenge: string; // base64url-encoded expected challenge
    expectedRpId: string; // expected relying party id
    expectedOrigin: (origin: string) => boolean; // function to validate origin
  }
}

export namespace Authentication {
  export interface Settings {
    rpId: string;
    allowCredentialIds?: string[];
  }

  export interface PublicKeyOptionsJSON {
    challenge: string;
    rpId: string;
    userVerification: UserVerificationRequirement;
    timeout: number;
    allowCredentials?: { type: "public-key"; id: string }[];
  }

  export interface AssertionJSON {
    credentialId: string;
    credentialType: string;
    clientDataJSON: string;
    authenticatorData: string;
    signature: string;
    userHandle?: string;
  }

  export interface VerificationOptions {
    expectedChallenge: string; // base64url-encoded expected challenge
    expectedRpId: string; // expected relying party id
    expectedOrigin: (origin: string) => boolean; // function to validate origin
  }
}
