import { serve } from "https://deno.land/std@0.155.0/http/server.ts";
import { serveFile } from "https://deno.land/std@0.155.0/http/file_server.ts";
import { resolve } from "https://deno.land/std@0.156.0/path/mod.ts";
import config from "./config.ts";
import routes from "./routes.ts";

function handler(request: Request): Promise<Response> {
  const url = new URL(request.url);
  const publicRoot = url.pathname.startsWith("/lib") ? "/" : "/client";
  const publicFile = url.pathname === "/" ? "/index.html" : url.pathname;

  switch (url.pathname) {
    case "/user/check":
      return routes.checkUser(request);
    case "/user":
      return routes.user(request);
    case "/email/authentication":
      return routes.emailAuthentication(request);
    case "/webauthn/registration":
      return routes.webAuthnRegistration(request);
    case "/webauthn/authentication":
      return routes.webAuthnAuthentication(request);
    default:
      return serveFile(request, resolve(`${Deno.cwd()}/..${publicRoot}${publicFile}`));
  }
}

console.log(`Deno running at: http://localhost:${config.port}/`);
await serve(handler, { port: config.port });
